import React, {useState, useEffect} from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleOrdinal,  format, extent  } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';

const width = 960;
const height = 500;
const margin = {top: 50, right:200, bottom:70, left: 100};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 50;
const tickOffset = 5;
const circleRadius = 7;


const App = () => {
  
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }
    
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
	const xValue = d => d.sepal_length;
  const xAxisLabel = 'sepal length';
 
	const yValue = d => d.sepal_width;
  const yAxisLabel = 'sepal width';

	const colorValue = d => d.species;
  const colorLegendLabel = 'Species';

	const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G','B')
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);
  
  const colorScale = scaleOrdinal()
  	.domain(data.map(colorValue))
  	.range([`#E6842A`,`#137B80`,`#8E6C8A`]);
  
//  console.log("X Scale Ticks:", xScale.ticks(5));
  //  console.log("Y Scale Ticks:", yScale.domain());
  
  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisBottom 
          xScale={xScale} 
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={tickOffset}
          />
        <text 
          className="axis-label" 
          x={innerWidth / 2} 
          y={innerHeight + xAxisLabelOffset} 
          textAnchor="middle">
          {xAxisLabel}
        </text>
        <AxisLeft 
          yScale={yScale} 
          innerWidth={innerWidth} 
          />
        <text 
          className="axis-label" 
          textAnchor="middle"
          transform={` translate(${-yAxisLabelOffset},${innerHeight / 2}),rotate(-90)`}
          >
          {yAxisLabel}
        </text>
        <g transform={`translate(${innerWidth + 50}, 60)`}>
          <text 
            x={35}
            y={-25}
            className="axis-label" 
            textAnchor="middle">
            {colorLegendLabel}
        </text>
          <ColorLegend 
            tickSpacing = {20} 
            tickSize = {10} 
            tickTextOffset = {15}
            tickSize={circleRadius}
            colorScale={colorScale} />
        </g>
        <Marks 
          data={data}
        	xValue={xValue}
          xScale={xScale} 
          yValue={yValue}
          yScale={yScale}
          colorValue={colorValue}
          colorScale={colorScale}
          tooltipFormat={xAxisTickFormat}
          circleRadius={circleRadius}
          />
      </g>
    </svg>
  );
};


const rootElement = document.getElementById('root');
  
ReactDOM.render(<App />, rootElement);